#include <QtGui>
#include <fileref.h>
#include <tag.h>
#include <iostream>

using namespace std;

void fileListR(const QString &_name, QStringList *_list, QStringList *_processedSLinks);
void parseFolder4MediaFiles(QString _folder, QStringList *_list, QStringList *_processedSLinks);

void fileList(const QStringList &_source, QStringList *_dest) {
    QStringList processedSLinks;
    QStringList::const_iterator it;
    for (it = _source.constBegin(); it != _source.constEnd(); ++it)
        fileListR(*it, _dest, &processedSLinks);
}

void parseFolder4MediaFiles(QString _folder, QStringList *_list, QStringList *_processedSLinks) {
    QStringList nameFilters;
    nameFilters << "*.mp3" << "*.ogg" << "*.flac";
    QDir dir(_folder);
    dir.setFilter(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Files);
    dir.setNameFilters(nameFilters);
    QStringList dirEntries = dir.entryList();

    QStringList::const_iterator it;
    QString prefix = _folder;
    if (!prefix.endsWith('/'))
        prefix.append('/');
    for (it = dirEntries.constBegin(); it != dirEntries.constEnd(); ++it) {
        fileListR(prefix + *it, _list, _processedSLinks);
    }
}

void fileListR(const QString &_name, QStringList *_list, QStringList *_processedSLinks) {
    QFileInfo fileInfo(_name);
    if (fileInfo.isFile()) {
        _list->append(_name);
    } else {
        if (fileInfo.isDir() || (fileInfo.isDir() && fileInfo.isSymLink() && !_processedSLinks->contains(_name))) {
            if (fileInfo.isSymLink())
                _processedSLinks->append(_name);
            parseFolder4MediaFiles(_name, _list, _processedSLinks);
        }
    }
}

TagLib::String formatSeconds(int seconds) {
    char secondsString[3];
    sprintf(secondsString, "%02i", seconds);
    return secondsString;
}

class TagReader : public QThread {
// Q_OBJECT
public:
    TagReader(QStringList file_list) : QThread(0), mFileList(file_list) {
        static int ins = 0;
        instance = ++ins;
    }
    ~TagReader() {}
protected:
    void run();
private:
    int instance;
    QStringList mFileList;
};

void TagReader::run() {
    QString file;
    int entry_no = 1;
    foreach(file, mFileList) {
        TagLib::FileRef f(file.toLatin1(), true);
        qDebug("Entry #%d of Instance %d", entry_no++, instance);
        if (!f.isNull() && f.tag()) {
            TagLib::Tag *tag = f.tag();
            cout << "Tag:" << tag->title()   <<tag->artist()  << tag->album()   << tag->year()    << tag->comment() << tag->track()   << tag->genre() << endl;
        }
        if (!f.isNull() && f.audioProperties()) {
            TagLib::AudioProperties *properties = f.audioProperties();
            cout << "Prop." << properties->bitrate() << properties->sampleRate() <<  properties->channels() << endl;
        }
    }
    qDebug("################################Thread#%d finished#####################################", instance);
}

class MyApplication : public QApplication {
public:
    MyApplication(int argc, char *argv[]) : QApplication(argc, argv) {
        cout << "Creating list of files..." << endl;
        QStringList dir_list;
        dir_list /*<< "/songs1" */<< "/songs2";
        QStringList file_list1;
        fileList(dir_list, &file_list1);
        QStringList file_list2;
        fileList(dir_list, &file_list2);
        cout << "List of files created. File found: " << file_list1.count() << endl;
        t1 = new TagReader(file_list1);
        t2 = new TagReader(file_list2);
        t1->start();
        t2->start();
    }
    ~MyApplication() {
        t1->wait();
        delete t1;
        t2->wait();
        delete t2;
    }
private:
    TagReader *t1;
    TagReader *t2;

};

int main(int argc, char *argv[]) {
    MyApplication app(argc, argv);
    return app.exec();
}
